@@ -88,7 +88,7 @@ class VMFixIPControllerApp(IPControllerApp):
         submitted upstream so we can remove this when incorporated into released IPython.
         """
         public_ips = []
-        vm_ifaces = set(["docker0", "virbr0"])  # VM/container interfaces we do not want
+        vm_ifaces = set(["docker0", "virbr0", "lxcbr0"])  # VM/container interfaces we do not want
 
         # list of iface names, 'lo0', 'eth0', etc.
         for iface in netifaces.interfaces():
@@ -101,6 +101,8 @@ class VMFixIPControllerApp(IPControllerApp):
                         continue
                     if not (iface.startswith('lo') or addr.startswith('127.')):
                         public_ips.append(addr)
+                        # pick first valid address for each interface to avoid double bound addresses
+                        break
         public_ips = uniq_stable(public_ips)
         return public_ips[-1]
 
@@ -848,7 +850,16 @@ def cluster_view(scheduler, queue, num_jobs, cores_per_job=1, profile=None,
     try:
         client = None
         slept = 0
-        while not _is_up(url_file, num_jobs):
+        max_up = 0
+        up = 0
+        while not up == num_jobs:
+            up = _nengines_up(url_file)
+            if up < max_up:
+                print ("Engine(s) that were up have shutdown prematurely. "
+                       "Aborting cluster startup.")
+                _stop(profile, cluster_id)
+                sys.exit(1)
+            max_up = up
             time.sleep(delay)
             slept += delay
             if slept > max_delay:
@@ -860,12 +871,31 @@ def cluster_view(scheduler, queue, num_jobs, cores_per_job=1, profile=None,
             yield _get_balanced_blocked_view(client, retries)
     finally:
         if client:
-            client.close()
+            _shutdown(client)
         _stop(profile, cluster_id)
         if has_throwaway:
             delete_profile(profile)
 
 
+def _nengines_up(url_file):
+    "return the number of engines up"
+    try:
+        client = Client(url_file, timeout=60)
+        up = len(client.ids)
+    # the controller isn't up yet
+    except iperror.TimeoutError:
+        return False
+        client.close()
+        return 0
+    # the JSON file is not available to parse
+    except IOError:
+        return False
+        client.close()
+        return 0
+    else:
+        return up
+
+
 def _get_balanced_blocked_view(client, retries):
     view = client.load_balanced_view()
     view.set_flags(block=True)
@@ -873,6 +903,11 @@ def _get_balanced_blocked_view(client, retries):
         view.set_flags(retries=int(retries))
     return view
 
+def _shutdown(client):
+    print "Sending a shutdown signal to the controller and engines."
+#    client.spin()
+#    client.shutdown(hub=True, block=False)
+    client.close()
 
 def _get_direct_view(client, retries):
     view = client[:]
